#include <lektor/common.h>
#include <lektor/bufferfd.h>

PRIVATE_FUNCTION ssize_t
__fill(struct bufferfd *bf)
{
    if (bf->len <= bf->pos) {
        ssize_t n = read(bf->fd, bf->buffer, BUFFER_MAX);

        if (n < 0)
            return -1;

        bf->len = (size_t)n;
        bf->pos = 0;
    }

    return (ssize_t)(bf->len - bf->pos);
}

ssize_t
bufferfd_bytes(struct bufferfd *bf, size_t n, uint8_t *res)
{
#pragma message(TODO "Repeat until n is reached")
    ssize_t r = __fill(bf);

    if (r < (ssize_t)n)
        return -1;

    memcpy(res, bf->buffer + bf->pos, n);
    bf->pos += n;
    return 1;
}

ssize_t
bufferfd_byte(struct bufferfd *bf, uint8_t *res)
{
    return bufferfd_bytes(bf, 1, res);
}

ssize_t
bufferfd_skip(struct bufferfd *bf, const size_t n)
{
    if (lseek(bf->fd, (off_t)(n - bf->len + bf->pos), SEEK_CUR) < 0)
        return -1;
    /* The call to lseek invalidated bf->buffer */
    bf->len = 0;
    __fill(bf);
    return 0;
}

ssize_t
bufferfd_seek(struct bufferfd *bf, size_t pos)
{
    if (lseek(bf->fd, (off_t)pos, SEEK_SET) < 0)
        return -1;
    /* The call to lseek invalidated bf->buffer */
    bf->pos = 0;
    bf->len = 0;
    return 0;
}
